#!/usr/bin/perl -w
############################## check_snmp_load #################
my $Version='1.0';
# Date : Oct 20 2014
# Author  : Nicolas Biersart
# Help : esia-sa.com
#################################################################

use strict;
use lib "/usr/local/esia/plugins/lib/perl"  ;
use Net::SNMP;
use Getopt::Long;
use RRDs;
use Data::Dumper;
use ESIA::SNMP;
use ESIA::PLUGIN;
my $PROGNAME='check_snmp_interface.pl';
my $PROGDESC = "Check server bandwith by SNMP";
my $TIMEOUT = $ESIA::PLUGIN::TIMEOUT;
my %ERRORS= %ESIA::PLUGIN::ERRORS;

my ($opt_h,$opt_V);
my $ifindex;
my $path='/tmp/';
my $warning='70';
my $critical='80';
my $speed=1000000000; # default 1000000000 = 1Gb/s
my $detectspeed="auto";
my $oid_outoctet='1.3.6.1.2.1.2.2.1.16';
my $oid_outHCoctet='1.3.6.1.2.1.31.1.1.1.10';
my $oid_inoctet='1.3.6.1.2.1.2.2.1.10';
my $oid_inHCoctet='1.3.6.1.2.1.31.1.1.1.6';
my $oid_speed='1.3.6.1.2.1.2.2.1.5';
my $oid_descr = "1.3.6.1.2.1.2.2.1.2";
my $oid_operstatus = "1.3.6.1.2.1.2.2.1.8";
my $oid_counter_type='32bit';
my $oid_highspeed='1.3.6.1.2.1.31.1.1.1.15';

sub print_usage() {
	print "Usage: \n";
	print "  $PROGNAME -H <HOSTNAME> [-C <community>] -i ifindex [-w warning] [-c critical]\n";
	print "  $PROGNAME [-h | --help]\n";
	print "  $PROGNAME [-V | --version]\n";
}
sub calc_bandwith_usage
{
	my $filename =  $_[0];
	my $actual_outoctet =  $_[1];
	my $actual_inoctet =  $_[2];
	my $timestamp = time();
	my $out_ocps=0;
	my $in_ocps=0;
	my $out_oc=0;
	my $in_oc=0;
	my $inter=0;
	#print "$filename\n";
	#on test si le fichier RRD existe
	if (! -e $filename)
	{
		open(FILECOUNTER,"+>$filename");
		print FILECOUNTER "$timestamp:$actual_outoctet:$actual_inoctet: ";
		close(FILECOUNTER);
	}
	else
	{
		open(FILECOUNTERS,"+<$filename") or die "Can't open $filename: $!";
		my $saveline="";
		while (my $line = <FILECOUNTERS>)
		{
			my @value = split(':',$line);
			$inter = ($timestamp -$value[0] );
			if($inter > 10)
			{
				$out_oc=($actual_outoctet -$value[1]);
				$in_oc=($actual_inoctet - $value[2]);

				if($out_oc>=0 && $in_oc >=0)
				{
					$out_ocps = sprintf("%.3f", ($actual_outoctet-$value[1])/$inter);
					$in_ocps = sprintf("%.3f", ($actual_inoctet-$value[2])/$inter);
				}
				last;
			}
			else
			{
				$saveline = $line;
			}

		}
		seek (FILECOUNTERS,0,0);
		print FILECOUNTERS "$timestamp:$actual_outoctet:$actual_inoctet:\n";
		print FILECOUNTERS "$saveline";
		close(FILECOUNTERS);
	}
	return ($out_oc,$in_oc,$out_ocps,$in_ocps);
}
#
#MAIN
#
ESIA::PLUGIN::arm_end_signal($TIMEOUT);
Getopt::Long::Configure( "bundling" );
my ($snmp, $snmperror) = ESIA::SNMP::SessionAutoCommunityVersion(@ARGV);
my $hostname = $snmp->hostname();
GetOptions(
	"V"   => \$opt_V,	"version"    => \$opt_V,
	"h"   => \$opt_h,	"help"       => \$opt_h,
	"i=i" => \$ifindex,"ifindex=i" => \$ifindex,
	"w=i" => \$warning,"warning=i" => \$warning,
	"c=i" => \$critical,"critical=i" => \$critical,
	"d=s" => \$path,"directory=s" => \$path,
	"s=s" => \$detectspeed,"speed=s"=>\$detectspeed
);
# -h & --help print help
if ($opt_h || !defined($snmp))
{
	ESIA::SNMP::print_snmp_usage($PROGNAME,'-m namestorage');
	my @arr;
	push(@arr,["-i","--ifindex","ifindex of the interface" ]);
	push(@arr,["-d","--directory","directory for tmp file" ]);
	push(@arr,["-s","--speed","speed (default: auto)" ]);
	push(@arr,["-w","--warning","warning threshold in percent (default: 80)" ]);
	push(@arr,["-c","--critical","critical threshold in percent(default: 90) "]);
	ESIA::SNMP::print_snmp_help($PROGNAME,$PROGDESC,\@arr);
	exit $ERRORS{'OK'};
}

#on interroge en SNMP pour récupérer les informations
my $result = ESIA::SNMP::get($snmp,"$oid_descr.$ifindex");
if (!defined($result))
{
    printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
    $snmp->close;
    exit $ERRORS{"UNKNOWN"};
}


my $name =  $result;
my $la =chop($name);
if($la =~ m/[0-9a-zA-Z]$/)
{
        chomp($la);
      $name = $name.$la;
}

my $defaultspeed='';
#récupération de la vitesse de l'interface
if($detectspeed eq 'auto')
{
    $result = ESIA::SNMP::get($snmp,"$oid_speed.$ifindex");
	if($result!=0)
	{
		if($speed > 4294967200)
		{
			$result = ESIA::SNMP::get($snmp,"$oid_highspeed.$ifindex");
			$speed = $result * 1000000;
		}
	}
	else
	{
		$defaultspeed='(speed recovery failed, use default speed)';
	}
}
else
{
	$speed = $detectspeed*1000*1000 ;
}

#récupération du nombre d'octet en sortie avec le counter64 (ifHCOutOctets) si échoué utilise le counter32 (1.3.6.1.2.1.2.2.1.16)
#récupère le nombre d'octet en entré suivant la valeur du dessus
my $outoctet = ESIA::SNMP::get($snmp,"$oid_outHCoctet.$ifindex");
my $inoctet = 0;
if (!defined($outoctet))
{
    $outoctet = ESIA::SNMP::get($snmp,"$oid_outoctet.$ifindex");
    $inoctet = ESIA::SNMP::get($snmp,"$oid_inoctet.$ifindex");
}
#certain implémente un counter64 mais le laisse à 0 (ex: certain switch TP-LINK)
elsif($outoctet==0)
{
    $outoctet = ESIA::SNMP::get($snmp,"$oid_outoctet.$ifindex");
    $inoctet = ESIA::SNMP::get($snmp,"$oid_inoctet.$ifindex");
}
else
{
	#OK le noeud est full compatible avec les compteurs 64 bit
	$inoctet = ESIA::SNMP::get($snmp,"$oid_inHCoctet.$ifindex");
	$oid_counter_type='64bit';
}

my $timestamp = time();
my $filename = $path.$hostname.'_'.$ifindex.".csv";

#calcul de la bande passante utilisée
my ($out,$in,$outkbs,$inkbs) = calc_bandwith_usage($filename,$outoctet,$inoctet);

$in = $inkbs*8;
$out = $outkbs*8;
#calcul de congestion
if($in > $speed)
{
        #print "CONGESTION\n";
        my $maxspeed=0;
        if($speed < 100000000)#100Mbs
        {
            $maxspeed = $speed + ($speed/2);
        }
        elsif($speed < 1000000000)#1000Mbs
        {
            $maxspeed = $speed + ($speed/10);
        }
        elsif($speed < 10000000000)#10000Mbs
        {
            $maxspeed = $speed + ($speed/100);
        }
        # print "CONGESTION: $in < $maxspeed $speed?\n";
        if($in <= $maxspeed)
        {
         #   print "CONGESTION: mise au maximun\n";
            $in = $speed;
            $inkbs = $speed;
        }
}
if($out > $speed)
{
    my $maxspeed=0;
    if($speed < 100000000) #100Mbs
    {
        $maxspeed = $speed + ($speed/2);
    }
    elsif($speed < 1000000000)#1000Mbs
    {
        $maxspeed = $speed + ($speed/10);
    }
    elsif($speed < 10000000000)#10000Mbs
    {
        $maxspeed = $speed + ($speed/100);
    }
    if($out <= $maxspeed)
    {
        $out = $speed;
        $outkbs = $speed;
    }
}
$inkbs = sprintf("%.2f", ($inkbs/1000)*8);
$outkbs = sprintf("%.2f", ($outkbs/1000)*8);

my $intunity ="kb/s";
my $outunity="kb/s";
# on regarde l'unité d'affiche en kb/s ou en Mb/S
if( int($inkbs/1000)>0)
{
	$inkbs = sprintf("%.2f", $inkbs/1000);
	$intunity ="Mb/s";
}
if( int($outkbs/1000)>0)
{
    $outkbs = sprintf("%.2f", $outkbs/1000);
    $outunity ="Mb/s";
}

# teste pour savoir si on est en warning ou en critique.
my $printspeed = $speed/1000/1000;
my $warningspeed = ($speed/100)*$warning;
my $criticalspeed = ($speed/100)*$critical;
my $graphdata = "|data:in=".$in."data:out=$out:warning:$warningspeed:critical:$criticalspeed";
if($in >= $warningspeed)
{
	if($in >= ($speed/100*$critical))
	{
		print "CRITICAL $name: in=$inkbs $intunity > $critical% out=$outkbs $outunity speed: $printspeed Mb/s Counter: $oid_counter_type $graphdata\n";
		exit $ERRORS{"CRITICAL"};
	}
	else
	{
		print "WARNING $name: in=$inkbs $intunity > $warning% out=$outkbs $outunity speed: $printspeed Mb/s Counter: $oid_counter_type $graphdata\n";
		exit $ERRORS{"WARNING"};
	}
}
elsif($out >= ($speed/100*$warning))
{
	if($out >= ($speed/100*$critical))
	{
		print "CRITICAL $name: in=$inkbs $intunity out=$outkbs $outunity  > $critical% speed: $printspeed Mb/s Counter: $oid_counter_type $graphdata\n";
		exit $ERRORS{"CRITICAL"};
	}
	else
	{
		print "WARNING $name: in=$inkbs $intunity out=$outkbs $outunity > $warning% speed: $printspeed Mb/s Counter: $oid_counter_type $graphdata\n";
		exit $ERRORS{"WARNING"};
	}
}
else
{
	print "OK $name: in=$inkbs $intunity out=$outkbs $outunity speed: $printspeed Mb/s $defaultspeed Counter: $oid_counter_type $graphdata\n";
	exit $ERRORS{"OK"};
}
