#!/usr/bin/perl
use strict;
use warnings;
use lib "/usr/local/esia/plugins/lib/perl"  ;
use vars qw($PROGNAME);
use Net::SNMP;
use Math::Round;
use Getopt::Long;
use ESIA::SNMP;
use ESIA::PLUGIN;

#nom du programme & description
my $PROGNAME = "check_snmp_linux_io.pl";
my $PROGDESC = "Linux IO Disks By SNMP";
my $PROGVERSION = "1.0";
#Chargement du timer par défaut et des erreurs
# a installer apt install libmath-round-perl
my $TIMEOUT = $ESIA::PLUGIN::TIMEOUT;
my %ERRORS= %ESIA::PLUGIN::ERRORS;

ESIA::PLUGIN::arm_end_signal($TIMEOUT);

Getopt::Long::Configure( "bundling" );
my ($opt_h,$opt_V);
my $community = "public";
my $snmp_version = 1;
my $warning=60;
my $critical=70;
my $return =0;
my $path = '/tmp/linuxio/';

my $oid_sysObjectOID                = "1.3.6.1.2.1.1.2.0";

my $oid_linux_disk_name = '1.3.6.1.4.1.2021.13.15.1.1.2'; 
my $oid_linux_disk_load_5min = '1.3.6.1.4.1.2021.13.15.1.1.10'; 
my $oid_linux_disk_read_byte = '1.3.6.1.4.1.2021.13.15.1.1.12';
my $oid_linux_disk_write_byte = '1.3.6.1.4.1.2021.13.15.1.1.13';

sub calc_bandwith_usage
{
	my $filename =  $_[0];
	my $actual_outoctet =  $_[1];
	my $actual_inoctet =  $_[2];
	my $timestamp = time();
	my $out_ocps=0;
	my $in_ocps=0;
	my $out_oc=0;
	my $in_oc=0;
	my $inter=0;
	#print "$filename\n";
	#on test si le fichier RRD existe
	if (! -e $filename)
	{
		open(FILECOUNTER,"+>$filename");
		print FILECOUNTER "$timestamp:$actual_outoctet:$actual_inoctet: ";
		close(FILECOUNTER);
	}
	else
	{
		open(FILECOUNTERS,"+<$filename") or die "Can't open $filename: $!";
		my $saveline="";
		while (my $line = <FILECOUNTERS>)
		{
			my @value = split(':',$line);
			$inter = ($timestamp -$value[0] );
			if($inter > 10)
			{
				$out_oc=($actual_outoctet -$value[1]);
				$in_oc=($actual_inoctet - $value[2]);

				if($out_oc>=0 && $in_oc >=0)
				{
					$out_ocps = sprintf("%.3f", ($actual_outoctet-$value[1])/$inter);
					$in_ocps = sprintf("%.3f", ($actual_inoctet-$value[2])/$inter);
				}
				last;
			}
			else
			{
				$saveline = $line;
			}

		}
		seek (FILECOUNTERS,0,0);
		print FILECOUNTERS "$timestamp:$actual_outoctet:$actual_inoctet:\n";
		print FILECOUNTERS "$saveline";
		close(FILECOUNTERS);
	}
	return ($out_oc,$in_oc,$out_ocps,$in_ocps);
}
if (!-d "$path") 
{
    mkdir( $path ) or die "Couldn't create $path directory, $!";
}
#on récupère les paramètres SNMP
my ($snmp, $snmperror) = ESIA::SNMP::Session(@ARGV);
my $match;
GetOptions(
	"V"   => \$opt_V,	 "version"    => \$opt_V,
	"h"   => \$opt_h,	 "help"       => \$opt_h,
	"m=s" => \$match,	 "match=s" => \$match,
	"w=i" => \$warning,	 "warning=i" => \$warning,
	"c=i" => \$critical, "critical=i" => \$critical,
);

# -h & --help print help
if ($opt_h)
{
    #affiche l'aide par défaut
    ESIA::SNMP::print_snmp_usage($PROGNAME,'-T [fan|psu|temp] [options]');
    my @arr;
    push(@arr,["-w","--warning","warning threshold max in (degrees for temp, percent for cpu)" ]);
    push(@arr,["-c","--critical","critical threshold max in (degrees for temp, percent for cpu)"]);
    push(@arr,["","--show-ignore","Print also components/values ignored"]);
    ESIA::SNMP::print_snmp_help($PROGNAME,$PROGDESC,\@arr);
    exit $ERRORS{'OK'};
}

# -V & --version print version
if ($opt_V)
{
    print "$PROGNAME version $PROGVERSION\n";
    exit $ERRORS{'OK'};
}

#on vérifie qu'on a bien un objet SNMP
if (!defined($snmp))
{
	print ("UNKNOWN: SNMP configuration error: $snmperror\n");
	exit $ERRORS{'UNKNOWN'};
}

my $state = 'UNKNOWN';
my $output = "";

my $sysOid_value = ESIA::SNMP::get($snmp,$oid_sysObjectOID);
if( !defined( $sysOid_value ) )
{
	print ("UNKNOWN: SNMP no response. $snmperror\n");
	exit $ERRORS{'UNKNOWN'};
}
 my $result = $snmp->get_table( $oid_linux_disk_name);
my $graphdata="";
my $hostname = $snmp->hostname();

foreach my $key ( sort keys %$result)
{
    if($key=~/\.([0-9]*)$/)
    {
        my $state = 'OK';
        my $id = $1;
        my $name = $$result{"$key"};
        if( !defined($match) || $name =~ /$match/)
        {
	        my $filename = $path.$hostname.'_'.$name.".csv";
		    my $used = ESIA::SNMP::get($snmp,"$oid_linux_disk_load_5min.$id");
			my $read = ESIA::SNMP::get($snmp,"$oid_linux_disk_read_byte.$id");
			my $write = ESIA::SNMP::get($snmp,"$oid_linux_disk_write_byte.$id");
            my ($readN,$writeN,$readBs,$writeBs) = calc_bandwith_usage($filename,$read,$write);
            my $message = "< $warning%";
            if( $used < $warning )
            {
                $state = 'OK';
            }
            elsif( $used < $critical )
            {
                $state = 'WARNING';
                $message = "> $warning%";
            }
            else
            {
                $state = 'CRITICAL';
                $message = "> $critical%";
            }
	        my $readunity ="kb/s";
	        my $writeunity="kb/s";
            $readBs = sprintf("%.0f", $readBs);
            $writeBs = sprintf("%.0f", $writeBs);
            my $readHumanBs = sprintf("%.1f", $readBs/1024);
            my $writeHumanBs = sprintf("%.1f", $writeBs/1024);
	        # on regarde l'unité d'affiche en kb/s ou en Mb/S
	        if( int($readHumanBs/1024)>0)
	        {
		        $readHumanBs = sprintf("%.1f", $readHumanBs/1024);
		        $readunity ="MB/s";
	        }
	        if( int($writeHumanBs/1024)>0)
	        {
		        $writeHumanBs = sprintf("%.1f", $writeHumanBs/1024);
		        $writeunity ="MB/s";
	        }
            print "$state : $name IO used $used% ($message). Read: $readHumanBs $readunity, Write: $writeHumanBs $writeunity\n";
            $graphdata.="percent=$used;read=$readBs;write=$writeBs";
		    $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
            if( !defined($match))
            {
                print "OK : Select first partition\n";
                last;
            }
        }
    }
}
print "|$graphdata";
exit $return;

