#!/usr/bin/perl
#
use strict;
use warnings;
use lib "/usr/local/esia/plugins/lib/perl"  ;
use vars qw($PROGNAME);
use Net::SNMP;
use Getopt::Long;
use ESIA::SNMP;
use ESIA::PLUGIN;

#nom du programme & description
my $PROGNAME = "check_snmp_load.pl";
my $PROGDESC = "Check load by SNMP";
my $PROGVERSION = "1.0";
#Chargement du timer par défaut et des erreurs
# a installer apt install libjson-perl
my $TIMEOUT = $ESIA::PLUGIN::TIMEOUT;
my %ERRORS= %ESIA::PLUGIN::ERRORS;

#my $linux_load_table= "1.3.6.1.4.1.2021.10.1";   # net-snmp load table
my $oid_linux_load_name = "1.3.6.1.4.1.2021.10.1.2"; # text 'Load-1','Load-5', 'Load-15'
my $oid_linux_load_value = "1.3.6.1.4.1.2021.10.1.3"; # effective load table
my $oid_proc_load = "1.3.6.1.2.1.25.3.3.1.2"; # %time the proc was not idle over last minute

my $oid_as400_cpu = "1.3.6.1.4.1.2.6.4.5.1.0"; # AS400 CPU load (10000=100%);

ESIA::PLUGIN::arm_end_signal($TIMEOUT);
Getopt::Long::Configure( "bundling" );
my ($opt_h,$opt_V);
my $warning=80;
my $critical=90;
my $type="standard";
my ($snmp, $snmperror) = ESIA::SNMP::SessionAutoCommunityVersion(@ARGV);
GetOptions(
		"V"   => \$opt_V,	 "version"    => \$opt_V,
		"h"   => \$opt_h,	 "help"       => \$opt_h,
		"T=s" => \$type,	 "Type=s" => \$type,
		"w=s" => \$warning,	 "warning=s" => \$warning,
		"c=s" => \$critical, "critical=s" => \$critical,
	  );


# -h & --help print help
if ($opt_h || !defined($snmp))
{
    #affiche l'aide par défaut
	ESIA::SNMP::print_snmp_usage($PROGNAME,'-T stand');
	my @arr;
	push(@arr,["-T","--type","type of cpu test [standard,netsl] (default: standard)" ]);
	push(@arr,["-w","--warning","warning threshold in percent  (default: 80)" ]);
	push(@arr,["-c","--critical","critical threshold in percent (default: 90)"]);
	ESIA::SNMP::print_snmp_help($PROGNAME,$PROGDESC,\@arr);
	exit $ERRORS{'OK'};
}
# -V & --version print version
if ($opt_V)
{
	print "$PROGNAME version $PROGVERSION\n";
	exit $ERRORS{'OK'};
}
$warning =~ s/\%//g;
$critical =~ s/\%//g;
my $Sysoid_value = ESIA::SNMP::getManufacturerOID($snmp);
if($Sysoid_value==-1)
{
	print "UNKNOWN: Host not SNMP response $snmperror\n";
	exit $ERRORS{'UNKNOWN'};
}
my $return = 0;
if($type eq 'standard')
{
	my $result = $snmp->get_table( $oid_proc_load);
	my $load=0;
	my $nbrcpu = 0;
    foreach my $key ( keys %$result)
    {
    	$load=$load + $$result{$key};
    	$nbrcpu++;
	}
	if($nbrcpu == 0)
	{
		print "UNKNOWN: No CPU information found\n";
		exit $ERRORS{'UNKNOWN'};
	}
	else
	{
		$load = $load / $nbrcpu;
		$load = sprintf("%.1f",$load);
		my $state = "OK";
		my $message = "< $warning%";
		if($load < $warning)
		{
			$state = "OK";
		}
		elsif($load < $critical)
		{
			$state = "WARNING";
			$message = "> $warning%";
		}
		else
		{
			$state = "CRITICAL";
			$message = "> $critical%";
		}

		print "$state: $nbrcpu CPU, average load $load% $message\n";
		$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});

	}
}
elsif($type eq 'netsl')
{
	my @arr_warning  =split(/,/ , $warning);
	my @arr_critical =split(/,/ , $critical);
	my $nwarn = @arr_warning;
	my $ncrit = @arr_critical;
	if($nwarn != 3 || $ncrit !=3)
	{
	 	print "Bad parameter, there are not 3 warnings and critical !\n";
	 	exit $ERRORS{'UNKNOWN'};
	}
	my $result = $snmp->get_table( $oid_linux_load_name);
    my $i =0;
    foreach my $key ( keys %$result)
    {
		if($key=~/\.([0-9]*)$/)
		{
			my $id = $1;
    		my $name = $$result{$key};
    		my $state = 'OK';
    		my $load = ESIA::SNMP::get($snmp,"$oid_linux_load_value.$id");

		    my $message = "< ".$arr_warning[$i];
		    if($load < $arr_warning[$i])
		    {
			    $state = "OK";
		    }
		    elsif($load < $arr_critical[$i])
		    {
			    $state = "WARNING";
			    $message = "> ".$arr_warning[$i];
		    }
		    else
		    {
			    $state = "CRITICAL";
			    $message = "> ".$arr_critical[$i];
		    }
    		print "$state: $name min is $load $message\n";
		    $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
            $i++;

    	}
	}
}
elsif($type eq 'as400')
{
	my $load = ESIA::SNMP::get($snmp,"$oid_as400_cpu");
	$load/=100;
	$load = sprintf("%.1f",$load);
	my $state = "OK";
	my $message = "< $warning%";
	if($load < $warning)
	{
		$state = "OK";
	}
	elsif($load < $critical)
	{
		$state = "WARNING";
		$message = "> $warning%";
	}
	else
	{
		$state = "CRITICAL";
		$message = "> $critical%";
	}

	print "$state: CPU, average load $load% $message\n";
	$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
}
exit $return;
