#!/usr/bin/perl
#
use strict;
use warnings;
use lib "/usr/local/esia/plugins/lib/perl"  ;
use vars qw($PROGNAME);
use Net::SNMP;
use Getopt::Long;
use ESIA::SNMP;
use ESIA::PLUGIN;

#nom du programme & description
my $PROGNAME = "check_snmp_mem.pl";
my $PROGDESC = "Check linux memory by SNMP";
my $PROGVERSION = "1.0";
#Chargement du timer par défaut et des erreurs
my $TIMEOUT = $ESIA::PLUGIN::TIMEOUT;
my %ERRORS= %ESIA::PLUGIN::ERRORS;

my $oid_ram_free	= "1.3.6.1.4.1.2021.4.6.0";  # Real memory free
my $oid_ram_total	= "1.3.6.1.4.1.2021.4.5.0";  # Real memory total
my $oid_ram_cache   = "1.3.6.1.4.1.2021.4.15.0"; # Real memory cached
my $oid_swap_free	= "1.3.6.1.4.1.2021.4.4.0";  # swap memory free
my $oid_swap_total	= "1.3.6.1.4.1.2021.4.3.0";  # Swap memory total



ESIA::PLUGIN::arm_end_signal($TIMEOUT);
Getopt::Long::Configure( "bundling" );
my ($opt_h,$opt_V);
my $warning="80,30";
my $critical="90,50";


my ($snmp, $snmperror) = ESIA::SNMP::SessionAutoCommunityVersion(@ARGV);
GetOptions(
		"V"   => \$opt_V,	 "version"    => \$opt_V,
		"h"   => \$opt_h,	 "help"       => \$opt_h,
		"w=s" => \$warning,	 "warning=s" => \$warning,
		"c=s" => \$critical, "critical=s" => \$critical,
	  );


# -h & --help print help
if ($opt_h || !defined($snmp))
{
#affiche l'aide par défaut
	ESIA::SNMP::print_snmp_usage($PROGNAME);
	my @arr;
	push(@arr,["-w","--warning","warning threshold in percent for memory usage ram and swap (default: 80,80)" ]);
	push(@arr,["-c","--critical","critical threshold in percent for memory usage ram and swap (default: 90,90)"]);
	ESIA::SNMP::print_snmp_help($PROGNAME,$PROGDESC,\@arr);
	exit $ERRORS{'OK'};
}
# -V & --version print version
if ($opt_V)
{
	print "$PROGNAME version $PROGVERSION\n";
	exit $ERRORS{'OK'};
}
my @arr_warning  =split(/,/ , $warning);
my @arr_critical =split(/,/ , $critical);
my $nwarn = @arr_warning;
my $ncrit = @arr_critical;
if($nwarn != 2 || $ncrit !=2)
{
	print "Bad parameter, here are not 2 warnings and critical for the RAM and SWAP!\n";
    exit $ERRORS{'UNKNOWN'};
}

my $return = 0;
my $ram_free = ESIA::SNMP::get($snmp,"$oid_ram_free");
if(!defined($ram_free))
{
	print "UNKNOWN: Host not SNMP response $snmperror\n";
	exit $ERRORS{'UNKNOWN'};
}
my $ram_cache = ESIA::SNMP::get($snmp,"$oid_ram_cache");
my $ram_total = ESIA::SNMP::get($snmp,"$oid_ram_total");
my $ram_used = undef;
if(defined($ram_cache))
{
    $ram_used = $ram_total-$ram_free - $ram_cache;
}
else
{
    $ram_used = $ram_total-$ram_free;
}
my $percent_used = ($ram_used/$ram_total)*100;
$percent_used = sprintf("%.0f",$percent_used);
my $ram_used_mb = sprintf("%.0f",$ram_used/1024);
my $ram_total_mb = sprintf("%.0f",$ram_total/1024);
my $state = "OK";
my $message = "< ".$arr_warning[0];
if($percent_used < $arr_warning[0])
{
    $state = "OK";
}
elsif($percent_used < $arr_critical[0])
{
    $state = "WARNING";
    $message = ">".$arr_warning[0];
}
else
{
    $state = "CRITICAL";
    $message = "> ".$arr_critical[0];
}

print "$state: Ram : $percent_used% (".$ram_used_mb."MB/".$ram_total_mb."MB) $message%\n";
$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
my $swap_total = ESIA::SNMP::get($snmp,"$oid_swap_total");
my $graphdata="ram=$percent_used;ramByte=$ram_used;ramTotal=$ram_total;";
if($swap_total !=0)
{
    $state = "OK";
    $message = "< ".$arr_warning[1];
    my $swap_free = ESIA::SNMP::get($snmp,"$oid_swap_free");
    my $swap_used = $swap_total-$swap_free;
    my $swap_percent_used = ($swap_used/$swap_total)*100;
    $swap_percent_used = sprintf("%.0f",$swap_percent_used);
    my $swap_used_MB = sprintf("%.0f",$swap_used/1024);
    my $swap_total_MB = sprintf("%.0f",$swap_total/1024);

    if($swap_percent_used < $arr_warning[1])
    {
        $state = "OK";
    }
    elsif($swap_percent_used < $arr_critical[1])
    {
        $state = "WARNING";
        $message = ">".$arr_warning[1];
    }
    else
    {
        $state = "CRITICAL";
        $message = "> ".$arr_critical[1];
    }
    print "$state: Swap : $swap_percent_used% $message%\n";
    $graphdata.="swap=$swap_percent_used;swapByte=$swap_used;swapTotal=$swap_total;";
}
else
{
    print"OK: No swap on device\n";
}
print "|$graphdata\n";
$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
exit $return;
