#!/usr/bin/perl
#
use strict;
use warnings;
use lib "/usr/local/esia/plugins/lib/perl"  ;
use vars qw($PROGNAME);
use Net::SNMP;
use Getopt::Long;
use ESIA::SNMP;
use ESIA::PLUGIN;
use Encode qw(decode encode);

#nom du programme & description
my $PROGNAME = "check_snmp_storage.pl";
my $PROGDESC = "Check storage space by SNMP";
my $PROGVERSION = "1.0";
#Chargement du timer par défaut et des erreurs
# a installer apt install libjson-perl
my $TIMEOUT = $ESIA::PLUGIN::TIMEOUT;
my %ERRORS= %ESIA::PLUGIN::ERRORS;

my $oid_descr = '1.3.6.1.2.1.25.2.3.1.3';
my $oid_size = '1.3.6.1.2.1.25.2.3.1.5';
my $oid_used = '1.3.6.1.2.1.25.2.3.1.6';
my $oid_alloc_units= '1.3.6.1.2.1.25.2.3.1.4';



ESIA::PLUGIN::arm_end_signal($TIMEOUT);
Getopt::Long::Configure( "bundling" );
my ($opt_h,$opt_V);
my $warning=80;
my $critical=90;
my $match=".*";
my ($snmp, $snmperror) = ESIA::SNMP::SessionAutoCommunityVersion(@ARGV);

$snmp->translate( ['-octetstring' => 0]);

GetOptions(
		"V"   => \$opt_V,	 "version"    => \$opt_V,
		"h"   => \$opt_h,	 "help"       => \$opt_h,
		"m=s" => \$match,	 "match=s" => \$match,
		"w=s" => \$warning,	 "warning=s" => \$warning,
		"c=s" => \$critical, "critical=s" => \$critical,
	  );


# -h & --help print help
if ($opt_h || !defined($snmp))
{
    #affiche l'aide par défaut
	ESIA::SNMP::print_snmp_usage($PROGNAME,'-m namestorage');
	my @arr;
	push(@arr,["-m","--match","name of storage (regex compatible)" ]);
	push(@arr,["-w","--warning","warning threshold in percent for used space (default: 80)" ]);
	push(@arr,["-c","--critical","critical threshold in percent for used space (default: 90) "]);
	ESIA::SNMP::print_snmp_help($PROGNAME,$PROGDESC,\@arr);
	exit $ERRORS{'OK'};
}
# -V & --version print version
if ($opt_V)
{
	print "$PROGNAME version $PROGVERSION\n";
	exit $ERRORS{'OK'};
}
$warning =~ s/\%//g;
$critical =~ s/\%//g;
my $Sysoid_value = ESIA::SNMP::getManufacturerOID($snmp);
if($Sysoid_value==-1)
{
	print "UNKNOWN: Host not SNMP response $snmperror\n";
	exit $ERRORS{'UNKNOWN'};
}
my $return = 0;
my $flag_match=0;
my $result = $snmp->get_table( $oid_descr);
my $graphdata="";
foreach my $key ( keys %$result)
{
#   my $name = $$result{$key};
    my $name = encode("UTF-8",$$result{$key});
    if($name =~/$match/)
    {
        $flag_match=1;
		if($key=~/\.([0-9]*)$/)
		{
            my $id = $1;
            my $used = ESIA::SNMP::get($snmp,"$oid_used.$id");
            my $size = ESIA::SNMP::get($snmp,"$oid_size.$id");
            my $unit = ESIA::SNMP::get($snmp,"$oid_alloc_units.$id");
            my $percent_used = ($used/$size)*100;
            $percent_used = sprintf("%.0f",$percent_used);
            my $used_byte = $used * $unit;
            my $size_byte = $size * $unit;
            my $used_MB =sprintf("%.0f",$used_byte/1024/1024);
            my $size_MB =sprintf("%.0f",$size_byte/1024/1024);
		    my $state = "OK";
		    my $message = "< $warning%";
		    if($percent_used < $warning)
		    {
			    $state = "OK";
		    }
		    elsif($percent_used < $critical)
		    {
			    $state = "WARNING";
			    $message = "> $warning%";
		    }
		    else
		    {
			    $state = "CRITICAL";
			    $message = "> $critical%";
		    }
            print "$state: $name: ".$percent_used."%"."used($used_MB"."MB/$size_MB"."MB) $message\n";
            $graphdata="used=$percent_used;usedByte=$used_byte;totalByte=$size_byte";
            $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
        }
    }
}
if($flag_match==0)
{
    print"UNKNOWN: partition $match not found\n";
    $return = $ERRORS{'UNKNOWN'};
}
else
{
    print"|$graphdata";
}
exit $return;

